# Simulation on a plant (MTG)

```@setup usepkg
using PlantBiophysics, PlantSimEngine, PlantMeteo, MultiScaleTreeGraph, PlantGeom
using CairoMakie, Dates

mtg = read_opf(joinpath(dirname(dirname(pathof(PlantBiophysics))), "test", "inputs", "scene", "opf", "coffee.opf"))

weather = read_weather(
    joinpath(dirname(dirname(pathof(PlantMeteo))), "test", "data", "meteo.csv"),
    :temperature => :T,
    :relativeHumidity => (x -> x ./ 100) => :Rh,
    :wind => :Wind,
    :atmosphereCO2_ppm => :Cₐ,
    date_format = DateFormat("yyyy/mm/dd")
)

models = read_model(joinpath(dirname(dirname(pathof(PlantBiophysics))), "test", "inputs", "models", "plant_coffee.yml"));

transform!(
    mtg,
    [:Ra_PAR_f, :Ra_NIR_f] => ((x, y) -> x + y * 1.2) => :Rᵢ, # This would be the incident radiation
    [:Ra_PAR_f, :Ra_NIR_f] => ((x, y) -> x + y) => :Rₛ,
    :Ra_PAR_f => (x -> x * 4.57) => :PPFD,
    (x -> 0.3) => :d,
    ignore_nothing = true
)

init_mtg_models!(mtg, models, length(weather))

run!(mtg, weather)

transform!(
    mtg,
    :Tₗ => (x -> x[1]) => :Tₗ_1,
    ignore_nothing = true
)
```

## Multi-scale Tree Graph

The Multi-scale Tree Graph, or MTG for short is a data structure that helps represent a plant topology, and optionally its geometry.

The OPF is a file format that stores an MTG with geometry onto the disk. Let's read an example OPF using `read_opf()`, a function from the `PlantGeom` package:

```@example usepkg
using PlantGeom
mtg = read_opf(joinpath(dirname(dirname(pathof(PlantBiophysics))), "test", "inputs", "scene", "opf", "coffee.opf"))
```

The result is an MTG defining the plant at several scales using a tree graph. You can read the introduction to the MTG from [MultiScaleTreeGraph.jl](https://vezy.github.io/MultiScaleTreeGraph.jl/stable/the_mtg/mtg_concept/)'s documentation if you want to understand how it works.

Now let's import the weather data:

```@example usepkg
using PlantBiophysics, PlantSimEngine, PlantMeteo

weather = read_weather(
    joinpath(dirname(dirname(pathof(PlantMeteo))), "test", "data", "meteo.csv"),
    :temperature => :T,
    :relativeHumidity => (x -> x ./ 100) => :Rh,
    :wind => :Wind,
    :atmosphereCO2_ppm => :Cₐ,
    date_format = DateFormat("yyyy/mm/dd")
)
```

And read the models associated to the MTG from a YAML file:

```@example usepkg
file = joinpath(dirname(dirname(pathof(PlantBiophysics))), "test", "inputs", "models", "plant_coffee.yml")
models = read_model(file)
```

Let's check which variables we need to provide for our model configuration:

```@example usepkg
to_initialize(models)
```

OK, only the `"Leaf"` component must be initialized before computation for the coupled energy balance, with the shortwave radiation `Rₛ`, the visible sky fraction seen by the object `sky_fraction`, the characteristic dimension of the object `d` and the Photosynthetically active Photon Flux Density `PPFD`. We are in luck, we used [Archimed-ϕ](https://archimed-platform.github.io/archimed-phys-user-doc/) to compute the radiation interception of each organ in the example coffee plant we are using. So the only thing we need is to transform the variables given by Archimed-ϕ in each node to compute the ones we need. We use `transform!` from `MultiScaleTreeGraph.jl` to traverse the MTG and compute the right variable for each node:

```@example usepkg
using MultiScaleTreeGraph

transform!(
    mtg,
    [:Ra_PAR_f, :Ra_NIR_f] => ((x, y) -> x + y * 1.2) => :Rᵢ, # This would be the incident radiation
    [:Ra_PAR_f, :Ra_NIR_f] => ((x, y) -> x + y) => :Rₛ,
    :Ra_PAR_f => (x -> x * 4.57) => :PPFD,
    (x -> 0.3) => :d,
    ignore_nothing = true
)
```

The design of `MultiScaleTreeGraph.transform!` is very close to the one adopted by `DataFrames`. It helps us compute new variables (or attributes) from others, modify their units or rename them. Here we compute `Rᵢ` and `Rₛ` from the sum of `Ra_PAR_f` (absorbed radiation flux in the PAR) and `Ra_NIR_f` (...in the NIR), `PPFD` from `Ra_PAR_f` using the conversion between ``W \cdot m^{2}`` to ``μmol \cdot m^{-2} \cdot s^{-1}``, and `d` using a constant value of 0.3 m. Note that `sky_fraction` is already computed for each node with the right units thanks to Archimed-ϕ, so no need to transform it.

Finally, we have to initialize the MTG with the models we want to use. We use `init_mtg_models!` from `PlantSimEngine` to do so:

```@example usepkg
init_mtg_models!(mtg, models, length(weather))
```

The function takes the MTG, the models, and the number of time-steps we want to simulate, and initializes the MTG with the models, and the variables needed for the simulation. The attributes can also be modified at this stage, for example by adding the variables that will be computed by the simulation, or by pre-allocating memory for the outputs.

Then `PlantBiophysics.jl` takes care of the rest and simulates the energy balance over each time-step:

```@example usepkg
run!(mtg, weather)
```

We can visualize the outputs in 3D using PlantGeom's `viz` function. To do so we have to extract the time step we want to use for coloring the organs. For example if we want to color the plant according to the value of the temperature from the first time-step, we would do:

```@example usepkg
transform!(
    mtg,
    :Tₗ => (x -> x[1]) => :Tₗ_1,
    ignore_nothing = true
)
```

And then actually plotting it:

```@example usepkg
f, ax, p = viz(mtg, color = :Tₗ_1)
colorbar(f[1, 2], p)
f
```
